/*
** Copyright (C) 2001 Sacha Faust <sacha@severus.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/*
 * Webspider.java
 *
 * Created on November 11, 2001, 2:26 AM
 */

package faust.sacha.web.bot.spider;

/**
 *
 * @author Sacha Faust
 * @version 1.0
 */
import org.apache.commons.httpclient.*;
import org.apache.commons.httpclient.methods.*;
import faust.sacha.web.data.*;
import faust.sacha.web.util.*;
import javax.swing.text.*;
import javax.swing.text.html.*;
import javax.swing.text.html.parser.*;
import java.util.*;
import java.io.*;

public class Webspider {

    /**
     */
    private Site m_site;
    private int m_requestMethodType;
    
    private Webspider() {
        // setting the default method type to METHOD_GET
        m_requestMethodType = WebGlobal.REQUEST_METHOD_GET;
    }

    public Webspider( Site toSpider ){
        this();
        m_site = toSpider;
    }
    
    public Webspider( Site toSpider, int requestMethod ){
        this(toSpider);
        m_requestMethodType = requestMethod;
    }
//------------------------------------------------------------------------------
    private HttpMethod getRequestMethodClass(){
        switch( m_requestMethodType ){
            case WebGlobal.REQUEST_METHOD_GET:
                return (new GetMethod());
            case WebGlobal.REQUEST_METHOD_HEAD:
                return (new HeadMethod());
            case WebGlobal.REQUEST_METHOD_POST:
                return (new PostMethod());
            case WebGlobal.REQUEST_METHOD_DELETE:
                return (new DeleteMethod());
            default:
                System.err.println("Webspider::getRequestMethodClass() : unkown method requestes" 
                                    + " setting to GET method");
                return (new GetMethod());
        }
    }
//------------------------------------------------------------------------------
    private void prepareRequestMethod( URLInfo url , HttpMethod requestMethod, boolean redirect ){

        requestMethod.setPath( url.getPath() );
        if( url.getQuery() != "" )
            requestMethod.setQueryString( url.getQuery() );
        
        //requestMethod.addRequestHeader("Accept", "text/html");
        requestMethod.addRequestHeader("User-Agent", WebGlobal.USER_AGENT);
        
        requestMethod.setFollowRedirects( redirect );
    }
//------------------------------------------------------------------------------    
    protected void setInfoToSite( URLData url ){
        
        switch( url.getURLType() ){
            case WebGlobal.URL_TYPE_FILE:
                if( WebGlobal.DEBUG )
                    System.err.println("Webspider::setInfoToSite() : URL_TYPE_FILE for : " + url.toString() );
                
                FileURL newFile = new FileURL(url);
                m_site.addFileToFolder(url.getFolder(), newFile);
                break;
            case WebGlobal.URL_TYPE_FOLDER:
                if( WebGlobal.DEBUG )
                    System.err.println("Webspider::setInfoToSite() : URL_TYPE_FOLDER for : " + url.toString() );
                
                m_site.addFolder( url.getFolder() );
                break;
            default:
                System.err.println("Webspider::setInfoToSite : CRITICAL : unknown URL TYPE SPECIFIED : " + url.getURLType() );
                System.err.println("aborting!Please report this");
                System.exit(1);
        }
    }
//------------------------------------------------------------------------------
    protected void processFileInfo( HTMLFile fileInfo, URLData url ){
        
        // setting URLData info
        url.addEmailList( fileInfo.getEmailFound() );
        url.addLinkList( fileInfo.getURLList() );
        processFileURLList( fileInfo, url );
        processEmailList( fileInfo );
        
    }
//------------------------------------------------------------------------------
    protected void processFileURLList( HTMLFile fileInfo, URLData url ){
        ArrayList urlList = null;
        //boolean stillGoOn = true;   // for illegalCaracters check
        String strURL = null;
        URLData newURLInfo = null;
        URLInfo urlBaseInfo = null;
        URLInfo wouldBeSameAs = null;

        /*
         * This fix some of the bug I've see when the code tries to fetch
         * java script.
         */
        //String illegalCaracters[] = { "(", ")", "{", "}" };
        
        urlList = fileInfo.getURLList();
        
        urlBaseInfo = new URLInfo( url.getBaseURL() );

        for( int i = 0; i < urlList.size(); i++ ){
            strURL = (String)urlList.get(i);
            
            /*
             * Checking for the presence of javascript() in the url
             *
             * Todo:
             *  manage javascript in URL in URLInfo
             *
             */
            if( strURL.indexOf("javascript") != -1 ){
                if( WebGlobal.DEBUG )
                    System.err.println("Webspider::processFileURLList() : found javascript in url : " + strURL );
                
                continue;
            }

            /*
             * TODO
             *  - see if this is really needed
             *
            for( int illegalCharIndex = 0; illegalCharIndex < illegalCaracters.length; illegalCharIndex++ ){
                URLInfo urlToCheck = new URLInfo(strURL);
                
                if( urlToCheck.getPath().indexOf(illegalCaracters[illegalCharIndex]) != -1 ){
                    stillGoOn = false;
                    break;
                }
            }
            if( !stillGoOn )
                continue;
                
            */
            
            /*
             *  Checking for creation of the same url as the parent url
             *
             *  Logic:
             *  Check if the combinaison the strURL and url.getBaseURL() would create the same
             *  link referenced to the current url. If it would, we simply skip it.
             *  This goes on for all the strURL that will be added to the parent URL.
             *  This is accomplish by the code using the variable wouldBeSameAs.
             */
             
            if( strURL.startsWith("_top") || strURL.startsWith("#") )       //getting rid of the reference to same doc
                continue;
            else if( strURL.startsWith("http://") || strURL.startsWith("https://") ){   // full URL
                
                /*
                 *  Checking if the url referencing itself
                 */
                if( strURL.compareToIgnoreCase(url.getURL()) == 0 ){
                    if( WebGlobal.DEBUG )
                        System.err.println("Webspider::processFileURLList() : strURL : " + strURL + " same as url : " + url.getURL() );
                    
                    continue;
                }
                
                newURLInfo = new URLData(strURL);
            }
            else if( strURL.lastIndexOf("/") == -1 ){           // file in same folder : Example : ulref = http://www.test.com/index.html  strURL = index2.html
                wouldBeSameAs = new URLData( strURL, urlBaseInfo );
                if( wouldBeSameAs.sameAs(url) ){
                    if( WebGlobal.DEBUG )
                        System.err.println("WebSpider::processFileURLList() : new url " + strURL + " with main url base " + urlBaseInfo.getURL() + " would be same as " + url.getURL() );
                    
                    continue;
                }
                
                newURLInfo = new URLData(strURL, url);
                if( WebGlobal.DEBUG )
                    System.err.println("WebSpider::processFileURLList-1 : relative url : urlRef : " + url.getURL() + " | strURL = " + strURL + " | newURL : " + newURLInfo.getURL() );
               
            }
            else if( (!strURL.startsWith("/")) && (strURL.lastIndexOf("/") != -1) ){    // folder/file or folder/file/ combo : the folder link is not preceded by a /
                wouldBeSameAs = new URLInfo( strURL, urlBaseInfo );
                if( wouldBeSameAs.sameAs(url) ){
                    if( WebGlobal.DEBUG )
                        System.err.println("WebSpider::processFileURLList() : new url " + strURL + " with main url base " + urlBaseInfo.getURL() + " would be same as " + url.getURL() );
                    
                    continue;
                }
                
                newURLInfo = new URLData(strURL, url);
                if( WebGlobal.DEBUG )
                    System.err.println("WebSpider::processFileURLList-2 : relative url : urlRef : " + url.getURL() + " | strURL = " + strURL + " | newURL : " + newURLInfo.getURL() );
               
            }
            else if( strURL.startsWith("/") ){  // Example : urlref = http://www.test.com strURL = /testing123

                wouldBeSameAs = new URLData( strURL, urlBaseInfo );
                if( wouldBeSameAs.sameAs(url) ){
                    if( WebGlobal.DEBUG )
                        System.err.println("WebSpider::processFileURLList() : new url " + strURL + " with main url base " + urlBaseInfo.getURL() + " would be same as " + url.getURL() );
                    
                    continue;
                }
                
                /*
                 * Here we create the new url with the parent base url since
                 * the link starts at the root.
                 */
                newURLInfo = new URLData( strURL, urlBaseInfo );

                if( WebGlobal.DEBUG )
                    System.err.println("WebSpider::processFileURLList-3 : relative url : urlRef : " + url.getURL() + " | strURL = " + strURL + " | newURL : " + newURLInfo.getURL() );
            
            }
            else{
                newURLInfo = new URLData(strURL);
            }
            
            if( !m_site.isURLAlreadyDone( (URLInfo)newURLInfo) ){
                m_site.addURLDone( (URLInfo)newURLInfo );
                
                /*
                 * Checking if the new url is in the same domain as the other one
                 * if not, we just continue to loop and we don't go fetch it
                 */
                if( url.getHost().compareTo(newURLInfo.getHost()) != 0 ){
                    if( WebGlobal.DEBUG )
                        System.err.println("Webspider::processFileURLList() : " + newURLInfo.getURL() + " not in same domain ");

                    m_site.addOutsideURL( (URLInfo)newURLInfo );
                    continue;
                }
                else if( newURLInfo.getPort() != url.getPort() ){
                    if( WebGlobal.DEBUG )
                        System.err.println("Webspider::processFileURLList() : " + newURLInfo.getURL() + " port : " + newURLInfo.getPort() );
                    
                    m_site.addOutsideURL( (URLInfo)newURLInfo );
                    continue;
                }
                
                getFileInfo(newURLInfo);
            }
            else{
                if( WebGlobal.DEBUG )
                    System.err.println("Webspider::processFileURLList() : url already done : " + newURLInfo.getURL() );
            }
            
        }
    }
//------------------------------------------------------------------------------
    protected void processEmailList( HTMLFile fileInfo ){
        ArrayList foundEmails = fileInfo.getEmailFound();
        
        for( int i = 0; i < foundEmails.size(); i++ ){
            String emailStr = (String)foundEmails.get(i);
            EmailURL newEmail = new EmailURL(emailStr);
            m_site.addEmail(newEmail);
        }
    }
//------------------------------------------------------------------------------    
    public void run(){
        
        URLData url = new URLData( m_site.getInfo() );
        setInfoToSite( url );
        m_site.addURLDone( (URLInfo)url );
        getFileInfo( url );
        
    }
//------------------------------------------------------------------------------
    public void setRequestMethod( int requestMethodType ){
        m_requestMethodType = requestMethodType;
    }
//------------------------------------------------------------------------------
    protected boolean getFileInfo( URLData url ){
        URLCont urlData = null;
        HttpMethod requestMethod = null;
        int requestCode;
        boolean allGood = true;        
          
        if( WebGlobal.DEBUG )
             System.err.println("\nWebspider::getFileInfo() : getting url : " + url.getURL() );
        
        urlData = new URLCont( (URLInfo)url );
        requestMethod = (HttpMethod)new HeadMethod();
        prepareRequestMethod( (URLInfo)url, requestMethod, WebGlobal.FOLLOW_REDIRECT );

        //urlData.setMethod(requestMethod);

        // get rid of expired cookies
        urlData.purgeCookies();
        
        try{
            urlData.open();
        }
        catch( IOException openIoEx ){
            System.err.println("Webspider::getFileInfo() : " + openIoEx);
            return false;
        }
            
        try{
            requestCode = requestMethod.execute(urlData.getState(), urlData);
            /*
             * TODO
             *  - add more return code 
             */
            switch( requestCode ){
                case 400:
                case 404:
                    allGood = false;
                    break;
            }
        }
        catch( HttpException httpEx ){
            System.err.println("Webspider::getFileInfo() : " + httpEx);
            allGood = false;
        }
        catch( IOException ioExRequest ){
            System.err.println("Webspider::getFile() : " + ioExRequest.toString());
            allGood = false;
        }
                
        //set the request Information
        url.setResponseInfo( requestMethod.getResponseHeaders() );
        if( !(url.getContentTypeInfo().compareToIgnoreCase("text/html") == 0) ){
            if( WebGlobal.DEBUG )
                System.err.println("WebSpider::getFileInfo() : Content-Type : " + url.getContentTypeInfo() + " on url : " + url.toString() );
                    
            allGood = false;
                
            // setting the url info
            setInfoToSite(url);
        }

        try{
            urlData.close();
        }
        catch( IOException closeIoEx ){
            System.err.println("WebSpider::getFileInfo() : " + closeIoEx );
        }
        
        // if everything is good, we pass the info to the handler
        if( allGood )
            getFileData(url);
        
        return allGood;
    }
//------------------------------------------------------------------------------    
    protected boolean getFileData( URLData url ){

        HTMLFile fileInfo = null;
        URLCont urlData = null;
        HTMLEditorKit.Parser parser = null;
        HTMLEditorKit.ParserCallback callback = null;
        HttpMethod requestMethod = null;
        boolean allGood = true;
        int requestCode;
        URLInfo tmpURL = null;
        String urlLocation = null;
        
        fileInfo = new HTMLFile();
        
        if( WebGlobal.DEBUG )
            System.err.println("WebSpider::getFileData() : getting url : " + url.getURL());
        
        try{
            ParserGetter kit = new ParserGetter();
            parser = kit.getParser();
            callback = fileInfo;
            
            try{
                urlData = new URLCont( (URLInfo)url );
                requestMethod = getRequestMethodClass();
                prepareRequestMethod( (URLInfo)url, requestMethod, WebGlobal.FOLLOW_REDIRECT );

                requestMethod.addRequestHeader("Accept", "text/html");
                //urlData.setMethod(requestMethod);
                
                // get rid of expired cookies
                urlData.purgeCookies();
                
                try{
                    urlData.open();
                }
                catch( IOException openIoEx ){
                    System.err.println("Webspider::getFileData() : " + openIoEx);
                    return false;
                }                
                try{
                    requestCode = requestMethod.execute(urlData.getState(), urlData);
                    /*
                     * TODO
                     *  - add more return code 
                     */
                    switch( requestCode ){
                        case 400:
                        case 404:
                            allGood = false;
                            break;
                        
                    }
                }
                catch( HttpException httpEx ){
                    System.err.println("Webspider::getFileData() : " + httpEx);
                    allGood = false;
                }
                catch( IOException ioExRequest ){
                    System.err.println("Webspider::getFileData() : " + ioExRequest.toString());
                    allGood = false;
                }
                
                //set the request Information
                url.setResponseInfo( requestMethod.getResponseHeaders() );
                //url.DEBUG_dump();
                
                if( !(url.getContentTypeInfo().compareToIgnoreCase("text/html") == 0) ){
                    //if( WebGlobal.DEBUG )
                        System.err.println("WebSpider::getFileData() : Content-Type : " + url.getContentTypeInfo() + " on url : " + url.toString() );
                    
                    allGood = false;
                }
                
                /*
                 * TODO :
                 *  - Add contentLocationInfo support
                 *
                 *Logic:
                 *  we check if the location header as data.
                 *  If there is data, we check if the location is the same
                 *  as the current url. We also check if this location is already
                 *  done. If both are false, we create a new URLData and send it to
                 *  getFile
                
                if( (urlLocation = url.getContentLocationInfo()) != null ){
                    tmpURL = new URLInfo(urlLocation);
                    
                    if( !tmpURL.sameAs((URLInfo)url) ){
                        if( !m_site.isURLAlreadyDone(tmpURL.toString()) ){
                            URLData newURL = new URLData(tmpURL);
                            setInfoToSite(newURL);
                        }
                    }
                }
                */
                // if everything is good, we pass the info to the handler
                if( allGood ){
                    InputStream in = requestMethod.getResponseBodyAsStream();
                    InputStreamReader r = new InputStreamReader(in);                                
                    parser.parse(r, callback,true);
                }
            }
            catch( IOException ioEx ){
                System.err.println("WebSpider::getFileData() : IOException : " + ioEx );
                allGood = false;
            }
        }
        catch( Exception e ){
            System.err.println("WebSpider::getFileData() : exception : " + e );
            e.printStackTrace();
            allGood = false;     
        }
        finally{
            try{
                urlData.close();
            }
            catch( IOException closeIoEx ){
                System.err.println("WebSpider::getFileData() : " + closeIoEx );
            }
        }
            
        // page 251 :Java Network Programming
        try{
            callback.flush();
        }
        catch( BadLocationException badLocE ){
            System.err.println("WebSpider::getFileData() : " + badLocE );
        }            
        setInfoToSite(url);
        processFileInfo( fileInfo, url );
        return allGood;
    }
    
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------    
    private void DEBUG_showUrlsDone(){
        System.out.println("URLS DONE:");
        ArrayList doneList = m_site.getUrlsDone();
        for( int i = 0; i < doneList.size(); i++ )
            System.out.println("url["+i+"] : " + (String)doneList.get(i));
    }    
//------------------------------------------------------------------------------
    private void DEBUG_dumpData(){
        Hashtable folderHash = m_site.getFolderHashTable();
        Enumeration folderKeys = folderHash.keys();
        int folderIndex = 0;
        
        System.out.println("\nSITE DATA");
        while( folderKeys.hasMoreElements() ){
            String key = (String)folderKeys.nextElement();
            ArrayList fileList = (ArrayList)folderHash.get(key);
            System.out.println("Folder[" + folderIndex + "] : " + key );
            for( int i = 0; i < fileList.size(); i++ ){
                FileURL file = (FileURL)fileList.get(i);
                System.out.println("\tFile[" + i + "] : " + file.getFileName());
                System.out.println("\t\tExtension : " + file.getExtension());
                System.out.println("\t\tServer : " + file.getServerInfo());
                System.out.println("\t\tContent-Length : " + file.getContentLengthInfoAsString());
                System.out.println("\t\tContent-Type : " + file.getContentTypeInfo());
                System.out.println("\t\tCookie : " + file.getCookieInfo());
                System.out.println("\t\tLast Modified : " + file.getLastModifiedInfo());
                System.out.println("\t\tLocation : " + file.getContentLocationInfo());
            }
            folderIndex++;
        }
    }
//------------------------------------------------------------------------------    
    public static void main( String args[] ){
        //URLInfo u = new URLInfo("http://jetson.wox.org");
        //URLInfo u = new URLInfo("http://bukakke");
        URLInfo u = new URLInfo("http://www.theredfly.com");
        Site s = new Site(u);
        Webspider t = new Webspider(s);
        
        t.run();
        
        t.DEBUG_showUrlsDone();
        t.DEBUG_dumpData();
        
        System.exit(0);
    }
}
