/*
** Copyright (C) 2001 Sacha Faust <sacha@severus.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
/*
 * Site.java
 *
 * Created on November 11, 2001, 1:39 AM
 */

package faust.sacha.web.data;

/**
 *
 * @author Sacha Faust
 * @version 1.0
 */
import faust.sacha.web.util.*;
import java.util.*;

public class Site {

    /**
     */
    private Hashtable m_siteFolders;
    private ArrayList m_urlsDone;
    private ArrayList m_emails;
    private URLInfo m_urlInfo;
    private ArrayList m_outsideURLs;

    private Site() {
        m_siteFolders = new Hashtable();
        m_urlsDone = new ArrayList(0);
        m_emails = new ArrayList(0);
        m_outsideURLs = new ArrayList(0);
    }

    public Site( URLInfo url ){
        this();
        m_urlInfo = url;
    }
    
    public Site( String url ){
        this();
        m_urlInfo = new URLInfo(url);
    }
//------------------------------------------------------------------------------
    protected boolean isEmailAlreadyFound( EmailURL emailToCheck ){
        EmailURL emailIndex = null;

        for( int i = 0; i < m_emails.size(); i++ ){
            emailIndex = (EmailURL)m_emails.get(i);
            if( emailIndex.sameAs(emailToCheck) )
                return true;
        }

        return false;
    }
//------------------------------------------------------------------------------
    protected boolean isFileAlreadyPresent( ArrayList elementList, String fileName ){
        FileURL fileIndex = null;
        for( int i = 0; i < elementList.size(); i++ ){
            fileIndex = (FileURL)elementList.get(i);
            
            if( fileIndex.getFileName().compareToIgnoreCase(fileName) == 0 )
                return true;
        }

        return false;
    }
//------------------------------------------------------------------------------    
    private void updateFileQueryData( String folder, FileURL fileData ){
        ArrayList fileList = getFolderElements(folder);
        FileURL fileToCheck = null;
        String queryToAdd = fileData.getQuery();
        
        if( queryToAdd == "" )
            return;
        /*
         *  we go over each files in the folder and compare only
         *  the filename to see if there is a match.
         *  if we find a match , we add the queryToAdd to the file
         */
        for( int i = 0; i < fileList.size(); i++ ){
            fileToCheck = (FileURL)fileList.get(i);
            
            // it's safe to only compare filenames 
            if( fileToCheck.getFileName().compareToIgnoreCase(fileData.getFileName()) == 0 ){
                fileToCheck.addQueryData( queryToAdd );
            }
        }
    }
//------------------------------------------------------------------------------
    public ArrayList getFolderElements( String folderName ){
        return (ArrayList)m_siteFolders.get(folderName);
    }
//------------------------------------------------------------------------------
    public String getSiteURL(){
        return m_urlInfo.getBaseURL();
    }
//------------------------------------------------------------------------------
    public int getPort(){
        return m_urlInfo.getPort();
    }
//------------------------------------------------------------------------------
    public boolean isSecure(){
        return m_urlInfo.isSecure();
    }
//------------------------------------------------------------------------------
    public String getDomain(){
        return m_urlInfo.getHost();
    }
//------------------------------------------------------------------------------    
    public void addFolder( String folderName ){

        if( !m_siteFolders.containsKey((Object)folderName) ){
            m_siteFolders.put( folderName, (Object)new ArrayList(0) );
            if( WebGlobal.DEBUG )
                System.err.println("Site::addFolder() : Adding folderKey " + folderName );
        }
        else{
            if( WebGlobal.DEBUG )
                System.err.println("Site::addFolder() : duplicate key " + folderName);
        }
    }
//------------------------------------------------------------------------------
    public void addFileToFolder( String folderName, FileURL fileToAdd ){
        ArrayList folderElements = null;
        FileURL presentFile = null;
        String queryToAdd = null;
        ArrayList fileList = null;
        
        /*
         *  if there is files in the folder
         *      we check if the file is already present in the folder
         *      if the file is present, we try to add new query data to it
         *      else we add the file to the folder
         *  else we try to add the folder and call back the same method
         */
        if( (folderElements = getFolderElements(folderName)) != null ){
            if( !isFileAlreadyPresent(folderElements, fileToAdd.getFileName()) )
                folderElements.add( (Object)fileToAdd );
            else{
                // if the file already exist, try to add some query data to it
                updateFileQueryData(folderName, fileToAdd);
            }
        }
        else{
            //the folder doesn't exist yet. create it and callback the same function
            if( WebGlobal.DEBUG )
                System.err.println("Site::addFileToFolder() : folder wasn't present and adding it for : " + folderName );

            addFolder(folderName);
            addFileToFolder(folderName, fileToAdd);
        }
    }
//------------------------------------------------------------------------------
    public void addURLDone( URLInfo urlToAdd ){
        m_urlsDone.add( (Object)urlToAdd );

        if( WebGlobal.DEBUG )
            System.err.println( "Site::addURLDone : added : " + urlToAdd.getURL() );
    }
//------------------------------------------------------------------------------
    public void addEmail( EmailURL emailToAdd ){
        if( !isEmailAlreadyFound(emailToAdd) )
            m_emails.add( (Object)emailToAdd );
    }
//------------------------------------------------------------------------------
    public ArrayList getUrlsDone(){
        return m_urlsDone;
    }
//------------------------------------------------------------------------------
    public ArrayList getEmailsFound(){
        return m_emails;
    }
//------------------------------------------------------------------------------
    /*
    public boolean isURLAlreadyDone( String urlToCheck ){
        String urlIndex = null;

        for( int i = 0; i < m_urlsDone.size(); i++ ){
            urlIndex = (String)m_urlsDone.get(i);
            if( urlIndex.compareToIgnoreCase(urlToCheck) == 0 )
                return true;
        }
        return false;
    }
     */
//------------------------------------------------------------------------------
    public boolean isURLAlreadyDone( URLInfo urlToCheck ){
        URLInfo urlIndex = null;
        String urlIndexQuery = null;
        String urlToCheckQuery = null;
        
        urlToCheckQuery = urlToCheck.getQuery();
        
        /*
         *  we go over all the urls that were done
         *  if we find a match on the basic url (ie : http://site.com/file withou anchor or query),
         *      we fetch the urldone query and store it in urlIndexQuery
         *      we see if botch urlTocheck and urlIndex as query data, 
         *          if not, we know we will point to the same data and return true
         *          else
         *              compare the query for both and if it's the same, we know we point to the data and return true
         *              else we know we never pointed to this data and return false
         */
        for( int i = 0; i < m_urlsDone.size(); i++ ){
            urlIndex = (URLInfo)m_urlsDone.get(i);
            
            if( urlIndex.getDirectURL().compareToIgnoreCase(urlToCheck.getDirectURL()) == 0 )
                return true;
            
            /*
            if( urlIndex.toString().compareToIgnoreCase(urlToCheck.toString()) == 0 ){
                // check if the query in both are the same
                urlIndexQuery = urlIndex.getQuery();
                
                //  if there is no query, we know it will points to the same data with or without anchor
                if( (urlToCheckQuery == "") && (urlIndexQuery == "") )
                    return true;
                else{
                    if( urlIndexQuery.compareToIgnoreCase(urlToCheckQuery) == 0 )       //compare the query part
                        return true;
                    else
                        return false;
                }       
            }
             */
        }
        return false;
    }
//------------------------------------------------------------------------------    
    public URLInfo getInfo(){
        return m_urlInfo;
    }
//------------------------------------------------------------------------------
    public Hashtable getFolderHashTable(){
        return m_siteFolders;
    } 
//------------------------------------------------------------------------------
    public Enumeration getFoldersNames() {
        return m_siteFolders.keys();
    }
//------------------------------------------------------------------------------
    public void addOutsideURL( URLInfo url ){
        m_outsideURLs.add( (Object)url );
    }
//------------------------------------------------------------------------------
    public ArrayList getOutsideURLs(){
        return m_outsideURLs;
    }
//------------------------------------------------------------------------------    
}
