/*
** Copyright (C) 2001 Sacha Faust <sacha@severus.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
/*
 * HTMLFile.java
 *
 * Created on June 16, 2001, 12:20 AM
 */

package faust.sacha.web.util;

import java.util.*;
import javax.swing.text.*;
import javax.swing.text.html.*;
import javax.swing.text.html.parser.*;
import java.io.*;

/**
 *
 * @author  Sacha Faust
 * @version 1.0
 */

public class HTMLFile extends HTMLParser {

    private ArrayList m_URLLinks;
    private ArrayList m_emailLinks;
    private ArrayList m_miscLinks;
    private String m_miscProtocols[] = {    "telnet://", 
                                            "ldap://",
                                            "ftp://"};       //Todo: handle this
    
    public HTMLFile(){
        super( new OutputStreamWriter(System.out) );
        m_URLLinks = new ArrayList();
        m_emailLinks = new ArrayList();
        m_miscLinks = new ArrayList();
    }
//------------------------------------------------------------------------------
    public void handleStartTag(HTML.Tag tag, MutableAttributeSet attributes, int position){

        Object attrib = null;
        String value = null;

        if( tag == HTML.Tag.A ){

            top_a:
            for( Enumeration attEnum = attributes.getAttributeNames(); attEnum.hasMoreElements(); ){
                try{
                    
                    attrib = attEnum.nextElement();
                    value = (String)attributes.getAttribute(attrib);
                    
                    if( value != null ){
                        if( attrib.toString().compareToIgnoreCase("href") == 0 ){
                            //quick cleanup
                            value = value.trim();
                            
                            //check if it's a empty link
                            if( value == "" )
                                continue top_a;
                            
                            if( value.startsWith("mailto:") ){
                                if( WebGlobal.DEBUG )
                                    System.err.println("HTMLFile::handleStartTag() : adding e-mail : " + value);

                                m_emailLinks.add((Object)value);
                                continue top_a;
                            }
                            else{
                                for( int protoIndex = 0; protoIndex < m_miscProtocols.length; protoIndex++ ){
                                    if( value.startsWith(m_miscProtocols[protoIndex]) ){
                                        if( WebGlobal.DEBUG )
                                            System.err.println("HTMLFile::handleStartTag() : adding misc link Attribute=" + attrib + " | value= " + value);

                                        m_miscLinks.add( (Object)value );
                                        continue top_a;
                                    }
                                }
                                //ok link
                                if( WebGlobal.DEBUG )
                                    System.err.println("HTMLFile::handleStartTag() : adding link : " + value);
                                
                                m_URLLinks.add( (Object)value );
                            }
                        }
                    }
                }
                catch( NoSuchElementException e ){
                    System.err.println("HTMLFile::handleStartTag() : A : " + e);
                }
            }
        }
    }
//------------------------------------------------------------------------------
    public void handleEndTag(HTML.Tag tag, int position){

        //DEBUG
        //DebugLog.println("</" + tag.toString() + ">" + "\n");
        //
        //System.out.println("end tag = " + tag);
    }
//------------------------------------------------------------------------------
    public void handleSimpleTag( HTML.Tag tag, MutableAttributeSet attributes, int position){
        
        Object attrib = null;
        String value = null;

        if( tag == HTML.Tag.FRAME ){
            
            top_frame:
            for( Enumeration attEnum = attributes.getAttributeNames(); attEnum.hasMoreElements(); ){
                try{                    
                    attrib = attEnum.nextElement();
                    value = (String)attributes.getAttribute(attrib);
                                        
                    if( value != null ){
                        //quick cleanup
                        value = value.trim();
                        
                        //check if it's a empty link
                        if( value == "" )
                            continue top_frame;
                        
                        if( attrib.toString().compareToIgnoreCase("src") == 0 ){   
                            for( int protoIndex = 0; protoIndex < m_miscProtocols.length; protoIndex++ ){
                                if( value.startsWith(m_miscProtocols[protoIndex]) ){
                                    if( WebGlobal.DEBUG )
                                        System.err.println("HTMLFile::handleSimpleTag() : FRAME : adding misc link Attribute=" + attrib + " | value= " + value);

                                        m_miscLinks.add( (Object)value );
                                        continue top_frame;
                                }
                            }
                            //ok link
                            if( WebGlobal.DEBUG )
                                System.err.println("HTMLFile::handleSimpleTag() : FRAME : adding link : " + value);

                            m_URLLinks.add( (Object)value );
                        }
                    }
                }
                catch( NoSuchElementException e ){
                    System.err.println("HTMLFile::handleSimpleTag() : FRAME : " + e);
                }
            }
        }
        else if( tag == HTML.Tag.LINK ){           
            top_link:
            for( Enumeration attEnum = attributes.getAttributeNames(); attEnum.hasMoreElements(); ){
                try{                    
                    attrib = attEnum.nextElement();
                    value = (String)attributes.getAttribute(attrib);
                                        
                    if( value != null ){
                        //quick cleanup
                        value = value.trim();
                        
                        //check if it's a empty link
                        if( value == "" )
                            continue top_link;
                        
                        if( attrib.toString().compareToIgnoreCase("href") == 0 ){   
                            if( value.startsWith("mailto:") ){
                                if( WebGlobal.DEBUG )
                                    System.err.println("HTMLFile::handleSimpleTag() : LINK : adding e-mail : " + value);

                                m_emailLinks.add((Object)value);
                                continue top_link;
                            }
                            else{
                                for( int protoIndex = 0; protoIndex < m_miscProtocols.length; protoIndex++ ){
                                    if( value.startsWith(m_miscProtocols[protoIndex]) ){
                                        if( WebGlobal.DEBUG )
                                            System.err.println("HTMLFile::handleSimpleTag() : LINK : adding misc link Attribute=" + attrib + " | value= " + value);

                                        m_miscLinks.add( (Object)value );
                                        continue top_link;
                                    }
                                }
                                //ok link
                                if( WebGlobal.DEBUG )
                                    System.err.println("HTMLFile::handleSimpleTag() : LINK : adding link : " + value);

                                m_URLLinks.add( (Object)value );
                            }
                        }
                    }
                }
                catch( NoSuchElementException e ){
                    System.err.println("HTMLFile::handleSimpleTag() : LINK : " + e);
                }
            }            
        }
        else if( tag == HTML.Tag.AREA ){
            top_area:
            for( Enumeration attEnum = attributes.getAttributeNames(); attEnum.hasMoreElements(); ){
                try{                    
                    attrib = attEnum.nextElement();
                    value = (String)attributes.getAttribute(attrib);
                                        
                    if( value != null ){
                        //quick cleanup
                        value = value.trim();
                        
                        //check if it's a empty link
                        if( value == "" )
                            continue top_area;
                        
                        if( attrib.toString().compareToIgnoreCase("href") == 0 ){   
                            if( value.startsWith("mailto:") ){
                                if( WebGlobal.DEBUG )
                                    System.err.println("HTMLFile::handleSimpleTag() : AREA : adding e-mail : " + value);

                                m_emailLinks.add((Object)value);
                                continue top_area;
                            }
                            else{
                                for( int protoIndex = 0; protoIndex < m_miscProtocols.length; protoIndex++ ){
                                    if( value.startsWith(m_miscProtocols[protoIndex]) ){
                                        if( WebGlobal.DEBUG )
                                            System.err.println("HTMLFile::handleSimpleTag() : AREA : adding misc link Attribute=" + attrib + " | value= " + value);

                                        m_miscLinks.add( (Object)value );
                                        continue top_area;
                                    }
                                }
                                //ok link
                                if( WebGlobal.DEBUG )
                                    System.err.println("HTMLFile::handleSimpleTag() : AREA : adding link : " + value);

                                m_URLLinks.add( (Object)value );
                            }
                        }
                    }
                }
                catch( NoSuchElementException e ){
                    System.err.println("HTMLFile::handleSimpleTag() : AREA : " + e);
                }
            }
        }
        else if( tag == HTML.Tag.BASE ){
            top_base:
            for( Enumeration attEnum = attributes.getAttributeNames(); attEnum.hasMoreElements(); ){
                try{                    
                    attrib = attEnum.nextElement();
                    value = (String)attributes.getAttribute(attrib);
                                        
                    if( value != null ){
                        //quick cleanup
                        value = value.trim();
                        
                        //check if it's a empty link
                        if( value == "" )
                            continue top_base;
                        
                        if( attrib.toString().compareToIgnoreCase("href") == 0 ){   
                            if( value.startsWith("mailto:") ){
                                if( WebGlobal.DEBUG )
                                    System.err.println("HTMLFile::handleSimpleTag() : BASE : adding e-mail : " + value);

                                m_emailLinks.add((Object)value);
                                continue top_base;
                            }
                            else{
                                for( int protoIndex = 0; protoIndex < m_miscProtocols.length; protoIndex++ ){
                                    if( value.startsWith(m_miscProtocols[protoIndex]) ){
                                        if( WebGlobal.DEBUG )
                                            System.err.println("HTMLFile::handleSimpleTag() : BASE : adding misc link Attribute=" + attrib + " | value= " + value);

                                        m_miscLinks.add( (Object)value );
                                        continue top_base;
                                    }
                                }
                                //ok link
                                if( WebGlobal.DEBUG )
                                    System.err.println("HTMLFile::handleSimpleTag() : BASE : adding link : " + value);

                                m_URLLinks.add( (Object)value );
                            }
                        }
                    }
                }
                catch( NoSuchElementException e ){
                    System.err.println("HTMLFile::handleSimpleTag() : BASE : " + e);
                }
            }            
        }
    }
//------------------------------------------------------------------------------
    public ArrayList getURLList(){
        return m_URLLinks;
    }
//------------------------------------------------------------------------------
    public ArrayList getEmailFound(){
        return m_emailLinks;
    }
//------------------------------------------------------------------------------
    public ArrayList getMiscFound(){
        return m_miscLinks;
    }
//------------------------------------------------------------------------------    

}

